@@ -32,7 +32,6 @@ import ctypes
 import json
 import multiprocessing
 import time
-from functools import partial
 from os.path import splitext
 from typing import List
 from typing import Union
@@ -67,6 +66,9 @@ from sonusai.utils import seconds_to_hms
 from sonusai.utils import to_numpy_array
 from sonusai.utils import trim_docstring
 
+# NOTE: multiprocessing dictionary is required for run-time performance; using 'partial' is much slower.
+MP_DICT = dict()
+
 
 # noinspection PyGlobalUndefined
 
@@ -114,10 +116,13 @@ def genmix(mixdb: dict,
            initial_i_frame_offset: int = 0,
            initial_o_frame_offset: int = 0) -> (np.ndarray, np.ndarray, np.ndarray, np.ndarray, dict):
     mixdb_out, total_samples = init(mixdb=mixdb, mixid=mixid, logging=logging)
-    compute_truth = compute_truth
-    compute_segsnr = compute_segsnr
-    target_audios = build_target_audio_db(mixdb=mixdb_out, show_progress=show_progress)
-    noise_audios = build_noise_audio_db(mixdb=mixdb_out, show_progress=show_progress)
+
+    MP_DICT['mixdb'] = mixdb_out
+    MP_DICT['total_samples'] = total_samples
+    MP_DICT['compute_truth'] = compute_truth
+    MP_DICT['compute_segsnr'] = compute_segsnr
+    MP_DICT['target_audios'] = build_target_audio_db(mixdb=mixdb_out, show_progress=show_progress)
+    MP_DICT['noise_audios'] = build_noise_audio_db(mixdb=mixdb_out, show_progress=show_progress)
 
     mp_mixture = multiprocessing.Array(ctypes.c_int16, total_samples)
     if compute_truth:
@@ -156,13 +161,7 @@ def genmix(mixdb: dict,
         progress = tqdm(total=len(mixdb_out['mixtures']), desc='genmix', disable=not show_progress)
         progress_needs_close = True
 
-    p_map(partial(_process_mixture,
-                  mixdb=mixdb_out,
-                  total_samples=total_samples,
-                  target_audios=target_audios,
-                  noise_audios=noise_audios,
-                  compute_truth=compute_truth,
-                  compute_segsnr=compute_segsnr),
+    p_map(_process_mixture,
           list(range(len(mixdb_out['mixtures']))),
           progress=progress,
           initializer=mp_init,
@@ -186,13 +185,14 @@ def genmix(mixdb: dict,
     return mixture, truth_t, target, noise, segsnr, mixdb_out
 
 
-def _process_mixture(mixid: int,
-                     mixdb: dict,
-                     total_samples: int,
-                     target_audios: list,
-                     noise_audios: list,
-                     compute_truth: bool,
-                     compute_segsnr: bool) -> None:
+def _process_mixture(mixid: int) -> None:
+    mixdb = MP_DICT['mixdb']
+    total_samples = MP_DICT['total_samples']
+    target_audios = MP_DICT['target_audios']
+    noise_audios = MP_DICT['noise_audios']
+    compute_truth = MP_DICT['compute_truth']
+    compute_segsnr = MP_DICT['compute_segsnr']
+
     mixture = to_numpy_array(mp_mixture, dtype=np.int16)
 
     truth_t = to_numpy_array(mp_truth_t, dtype=np.single)
