@@ -117,7 +117,6 @@ will find all .wav files in the specified directories and process them as target
 """
 import json
 import time
-from functools import partial
 from glob import glob
 from os import listdir
 from os.path import dirname
@@ -143,9 +142,10 @@ from sonusai import initial_log_messages
 from sonusai import logger
 from sonusai import update_console_handler
 from sonusai.mixture import apply_augmentation
-from sonusai.mixture import balance_classes
 from sonusai.mixture import estimate_audio_length
+from sonusai.mixture import generate_truth
 from sonusai.mixture import get_augmentations
+from sonusai.mixture import get_class_count
 from sonusai.mixture import get_class_weights_threshold
 from sonusai.mixture import get_next_noise
 from sonusai.mixture import get_total_class_count
@@ -160,6 +160,9 @@ from sonusai.utils import p_map
 from sonusai.utils import seconds_to_hms
 from sonusai.utils import trim_docstring
 
+# NOTE: multiprocessing dictionary is required for run-time performance; using 'partial' is much slower.
+MP_DICT = dict()
+
 
 def genmixdb(file: str = None,
              config: dict = None,
@@ -215,6 +218,17 @@ def genmixdb(file: str = None,
         logger.debug('Expanded list of noise augmentations:')
         logger.debug(expanded_noise_augmentations)
 
+    if logging:
+        logger.debug(f'SNRs: {config["noise_mix"]["snrs"]}\n')
+        logger.debug(f'Exhaustive noise: {config["noise_mix"]["exhaustive"]}\n')
+
+    noise_sets = len(noises) * len(noise_augmentations)
+    target_sets = len(targets) * len(target_augmentations) * len(config['noise_mix']['snrs'])
+    total_mixtures = noise_sets * target_sets
+    if logging:
+        logger.info('')
+        logger.info(f'Found {total_mixtures} mixtures to process')
+
     class_weights_threshold = get_class_weights_threshold(config)
 
     if config['truth_mode'] not in ['normal', 'mutex']:
@@ -242,97 +256,17 @@ def genmixdb(file: str = None,
     feature_samples = config['frame_size'] * decimation * stride
     feature_step_samples = config['frame_size'] * decimation * step
 
-    mixdb = {
-        'class_count':              list(),
-        'class_labels':             config['class_labels'],
-        'class_weights_threshold':  list(class_weights_threshold),
-        'feature':                  config['feature'],
-        'feature_samples':          feature_samples,
-        'feature_step_samples':     feature_step_samples,
-        'frame_size':               config['frame_size'],
-        'mixtures':                 list(),
-        'noise_mix':                {
-            'files':         noises,
-            'augmentations': noise_augmentations,
-            'snrs':          config['noise_mix']['snrs'],
-            'exhaustive':    config['noise_mix']['exhaustive'],
-        },
-        'num_classes':              config['num_classes'],
-        'target_augmentations':     target_augmentations,
-        'targets':                  targets,
-        'truth_mutex':              truth_mutex,
-        'truth_reduction_function': config['truth_reduction_function'],
-    }
-
-    # Read in all audio data beforehand to avoid reading it multiple times in the loop
-    if logging:
-        logger.debug('Reading noise audio')
-    raw_noise_audio = list()
-    for noise in noises:
-        raw_noise_audio.append(read_audio(name=noise['name']))
-
-    if logging:
-        logger.debug('Reading target audio')
-    raw_target_audio = list()
-    for target in targets:
-        raw_target_audio.append(read_audio(name=target['name']))
-
-    # Apply all target augmentations to all targets and generate initial class count
-    augmented_targets = []
-    for target_index, target in enumerate(targets):
-        for target_augmentation_index in range(len(target_augmentations)):
-            if not isinstance(target['truth_settings'], list):
-                target['truth_settings'] = [target['truth_settings']]
-
-            update_truth_settings(target['truth_settings'], config['truth_settings'])
-
-            for truth_setting in target['truth_settings']:
-                if not isinstance(truth_setting['index'], list):
-                    truth_setting['index'] = [truth_setting['index']]
-
-                if any(idx > max_class for idx in truth_setting['index']):
-                    raise SonusAIError('invalid truth index')
-
-            augmented_targets.append({
-                'target_file_index':         target_index,
-                'target_augmentation_index': target_augmentation_index,
-            })
-
-    progress = tqdm(total=len(augmented_targets), desc='genmixdb 1', disable=not show_progress)
-
-    augmented_targets = p_map(partial(process_target,
-                                      mixdb=mixdb,
-                                      raw_target_audio=raw_target_audio),
-                              augmented_targets,
-                              progress=progress)
-
-    progress.close()
-
-    # Balance class data
-    if config['class_balancing']:
-        augmented_targets = balance_classes(mixdb=mixdb,
-                                            augmented_targets=augmented_targets,
-                                            class_balancing_augmentation=config['class_balancing_augmentation'],
-                                            raw_target_audio=raw_target_audio)
-
-    noise_sets = len(noises) * len(noise_augmentations)
-    target_sets = len(augmented_targets) * len(config['noise_mix']['snrs'])
-    total_mixtures = noise_sets * target_sets
-    if logging:
-        logger.info('')
-        logger.info(f'Found {total_mixtures} mixtures to process')
-
     total_duration = 0
-    for augmented_target in augmented_targets:
-        length = int(targets[augmented_target['target_file_index']]['duration'] * sonusai.mixture.SAMPLE_RATE)
-        augmentation = target_augmentations[augmented_target['target_augmentation_index']]
-        if 'tempo' in augmentation:
-            length /= augmentation['tempo']
-        if length % feature_step_samples:
-            length += feature_step_samples - int(length % feature_step_samples)
-        total_duration += float(length) / sonusai.mixture.SAMPLE_RATE
-    total_duration *= len(config['noise_mix']['snrs'])
+    for duration in [sub['duration'] for sub in targets]:
+        for augmentation in target_augmentations:
+            length = int(duration * sonusai.mixture.SAMPLE_RATE)
+            if 'tempo' in augmentation:
+                length /= augmentation['tempo']
+            if length % feature_step_samples:
+                length += feature_step_samples - int(length % feature_step_samples)
+            total_duration += float(length) / sonusai.mixture.SAMPLE_RATE
     total_duration *= noise_sets
+    total_duration *= len(config['noise_mix']['snrs'])
 
     total_samples = total_duration * sonusai.mixture.SAMPLE_RATE
 
@@ -353,6 +287,48 @@ def genmixdb(file: str = None,
         logger.info(f'Feature samples:      {feature_samples} samples ({feature_ms} ms)')
         logger.info(f'Feature step samples: {feature_step_samples} samples ({feature_step_ms} ms)')
 
+    mixdb = {
+        'class_count':                  list(),
+        'class_labels':                 config['class_labels'],
+        'class_weights_threshold':      list(class_weights_threshold),
+        'feature':                      config['feature'],
+        'feature_samples':              feature_samples,
+        'feature_step_samples':         feature_step_samples,
+        'frame_size':                   config['frame_size'],
+        'mixtures':                     list(),
+        'noise_mix':                    {
+            'files':         noises,
+            'augmentations': noise_augmentations,
+            'snrs':          config['noise_mix']['snrs'],
+            'exhaustive':    config['noise_mix']['exhaustive'],
+        },
+        'num_classes':                  config['num_classes'],
+        'target_augmentations':         target_augmentations,
+        'targets':                      targets,
+        'truth_mutex':                  truth_mutex,
+        'truth_reduction_function':     config['truth_reduction_function'],
+        'class_balancing_augmentation': config['class_balancing_augmentation'],
+        'seed':                         config['seed'],
+    }
+
+    MP_DICT['mixdb'] = mixdb
+    MP_DICT['config'] = config
+
+    # Read in all audio data beforehand to avoid reading it multiple times in the loop
+    if logging:
+        logger.debug('Reading noise audio')
+    raw_noise_audio = list()
+    for noise in noises:
+        raw_noise_audio.append(read_audio(name=noise['name']))
+
+    if logging:
+        logger.debug('Reading target audio')
+    raw_target_audio = list()
+    for target in targets:
+        raw_target_audio.append(read_audio(name=target['name']))
+
+    MP_DICT['raw_target_audio'] = raw_target_audio
+
     # Get indices and offsets
     mixtures = [[] for _ in range(noise_sets)]
     n_id = 0
@@ -364,48 +340,46 @@ def genmixdb(file: str = None,
             noise_length = estimate_audio_length(audio_in=raw_noise_audio[noise_index],
                                                  augmentation=noise_augmentation,
                                                  length_common_denominator=1)
-            for target in augmented_targets:
-                target_index = target['target_file_index']
-                target_augmentation_index = target['target_augmentation_index']
-                target_augmentation = mixdb['target_augmentations'][target_augmentation_index]
+            for target_index, target in enumerate(targets):
+                for target_augmentation_index, target_augmentation in enumerate(target_augmentations):
+                    if not isinstance(target['truth_settings'], list):
+                        target['truth_settings'] = [target['truth_settings']]
 
-                for snr in config['noise_mix']['snrs']:
-                    mixtures[n_id][t_id] = {
-                        'target_file_index':         target_index,
-                        'noise_file_index':          noise_index,
-                        'noise_offset':              noise_offset,
-                        'target_augmentation_index': target_augmentation_index,
-                        'noise_augmentation_index':  noise_augmentation_index,
-                        'snr':                       snr,
-                        'target_snr_gain':           np.NaN,
-                        'noise_snr_gain':            np.NaN,
-                        'samples':                   target['samples'],
-                        'target_gain':               target['target_gain'],
-                        'class_count':               target['class_count'],
-                    }
-                    t_id += 1
+                    update_truth_settings(target['truth_settings'], config['truth_settings'])
 
-                    target_length = estimate_audio_length(audio_in=raw_target_audio[target_index],
-                                                          augmentation=target_augmentation,
-                                                          length_common_denominator=feature_step_samples)
-                    noise_offset = int((noise_offset + target_length) % noise_length)
+                    for truth_setting in target['truth_settings']:
+                        if not isinstance(truth_setting['index'], list):
+                            truth_setting['index'] = [truth_setting['index']]
+
+                        if any(idx > max_class for idx in truth_setting['index']):
+                            raise SonusAIError('invalid truth index')
+
+                    for snr in config['noise_mix']['snrs']:
+                        mixtures[n_id][t_id] = {
+                            'target_file_index':         target_index,
+                            'noise_file_index':          noise_index,
+                            'noise_offset':              noise_offset,
+                            'target_augmentation_index': target_augmentation_index,
+                            'noise_augmentation_index':  noise_augmentation_index,
+                            'snr':                       snr,
+                        }
+                        t_id += 1
+
+                        target_length = estimate_audio_length(audio_in=raw_target_audio[target_index],
+                                                              augmentation=target_augmentation,
+                                                              length_common_denominator=feature_step_samples)
+                        noise_offset = int((noise_offset + target_length) % noise_length)
 
             n_id += 1
 
-    progress = tqdm(total=total_mixtures, desc='genmixdb 2', disable=not show_progress)
-
     # Fill in the details
+    progress = tqdm(total=total_mixtures, desc='genmixdb', disable=not show_progress)
     for n_id in range(len(mixtures)):
-        augmented_noise_audio = apply_augmentation(
+        MP_DICT['augmented_noise_audio'] = apply_augmentation(
             audio_in=raw_noise_audio[mixtures[n_id][0]['noise_file_index']],
             augmentation=noise_augmentations[mixtures[n_id][0]['noise_augmentation_index']],
             length_common_denominator=1)
-        mixtures[n_id] = p_map(partial(_process_mixture,
-                                       mixdb=mixdb,
-                                       raw_target_audio=raw_target_audio,
-                                       augmented_noise_audio=augmented_noise_audio),
-                               mixtures[n_id],
-                               progress=progress)
+        mixtures[n_id] = p_map(_process_mixture, mixtures[n_id], progress=progress)
 
     progress.close()
 
@@ -528,10 +502,15 @@ def get_output_from_config(config: dict, config_name: str) -> str:
         raise SonusAIError(f'Error getting genmixdb base name: {e}')
 
 
-def _process_mixture(record: dict,
-                     mixdb: dict,
-                     raw_target_audio: list,
-                     augmented_noise_audio: np.ndarray) -> dict:
+def _process_target(record: dict) -> dict:
+    return process_target(record=record, mixdb=MP_DICT['mixdb'], raw_target_audio=MP_DICT['raw_target_audio'])
+
+
+def _process_mixture(record: dict) -> dict:
+    mixdb = MP_DICT['mixdb']
+    raw_target_audio = MP_DICT['raw_target_audio']
+    augmented_noise_audio = MP_DICT['augmented_noise_audio']
+
     target_index = record['target_file_index']
     noise_offset = record['noise_offset']
     target_augmentation_index = record['target_augmentation_index']
@@ -542,13 +521,31 @@ def _process_mixture(record: dict,
                                                 augmentation=target_augmentation,
                                                 length_common_denominator=mixdb['feature_step_samples'])
 
+    record['samples'] = len(augmented_target_audio)
+
+    # target_gain is used to back out the gain augmentation in order to return the target audio
+    # to its normalized level when calculating truth.
+    if 'gain' in target_augmentation:
+        record['target_gain'] = 10 ** (target_augmentation['gain'] / 20)
+    else:
+        record['target_gain'] = 1
+
+    truth_index = [sub['index'] for sub in mixdb['targets'][target_index]['truth_settings']]
+
+    truth = generate_truth(mixdb=mixdb,
+                           record=record,
+                           target=augmented_target_audio)
+
+    class_weights_threshold = get_class_weights_threshold(mixdb)
+
+    record['class_count'] = get_class_count(truth_index=truth_index,
+                                            truth=truth,
+                                            class_weights_threshold=class_weights_threshold)
+
     noise_segment, _ = get_next_noise(offset_in=noise_offset,
                                       length=len(augmented_target_audio),
                                       audio_in=augmented_noise_audio)
 
-    if len(augmented_target_audio) != record['samples']:
-        raise SonusAIError('Number of samples in mixture does not match initial value')
-
     if record['snr'] < -96:
         # Special case for zeroing out target data
         record['target_snr_gain'] = 0
