@@ -1,6 +1,14 @@
 import warnings
 
 import numpy as np
+from sklearn.metrics import average_precision_score
+from sklearn.metrics import confusion_matrix
+from sklearn.metrics import multilabel_confusion_matrix
+from sklearn.metrics import precision_recall_fscore_support
+from sklearn.metrics import roc_auc_score
+
+from sonusai import logger
+from sonusai.utils.reshape import reshape_truth_predict
 
 
 def one_hot(truth: np.ndarray,
@@ -41,56 +49,35 @@ def one_hot(truth: np.ndarray,
         rmse:    num_classes x 1             RMS error over all frames + timesteps, before threshold decision
         mavg:    3x8 averages                macro,micro,weighted [PPV, TPR, F1, FPR, ACC, mAP, mAUC, TPSUM]
     """
-    from sklearn.metrics import average_precision_score
-    from sklearn.metrics import confusion_matrix
-    from sklearn.metrics import multilabel_confusion_matrix
-    from sklearn.metrics import precision_recall_fscore_support
-    from sklearn.metrics import roc_auc_score
-
-    from sonusai import logger
-
     if truth.shape != predict.shape:
         logger.error('Shape of truth and predict are not equal')
         exit()
 
-    # truth, predict can be either frames x num_classes, or frames x timesteps x num_classes
-    # in binary case dim may not exist, detect this and set num_classes == 1
-    if truth.ndim == 3 or (truth.ndim == 2 and timesteps > 0):
-        if truth.ndim == 2:  # 2D but has timesteps = truth.shape[1]
-            num_classes = 1
-        else:
-            num_classes = truth.shape[2]  # frames = truth.shape[0], timesteps = truth.shape[1]
-        # reshape to remove timestep dimension
-        truth = np.reshape(truth, (truth.shape[0] * truth.shape[1], truth.shape[2]))
-        predict = np.reshape(predict, (predict.shape[0] * predict.shape[1], predict.shape[2]))
-    else:
-        if truth.ndim == 1:  # no timesteps dimension, = 0
-            num_classes = 1
-            truth = np.expand_dims(truth, 1)  # convert to 2D (F,1) if only 1 dim
-        else:
-            num_classes = truth.shape[1]  # 2D input frames x num_classes
-
-    # Support predict (and truth) when shape is (F,), just convert to (F,1)
-    if predict.ndim == 1:
-        predict = np.expand_dims(predict, 1)
+    truth, predict, num_classes = reshape_truth_predict(truth, predict, timesteps)
 
     # Regression metric root-mean-square-error always works
     rmse = np.sqrt(np.mean(np.square(truth - predict), axis=0))
 
     # Calc default predict decision thresholds based on mode
-    if not np.ndim(predict_thr) and predict_thr == 0:  # if scalar and 0, set defaults
+    if not np.ndim(predict_thr) and predict_thr == 0:
+        # if scalar and 0, set defaults
         if num_classes == 1:
-            predict_thr = np.atleast_1d(0.5)  # binary case default >= 0.5 which is equiv to argmax()
+            # binary case default >= 0.5 which is equiv to argmax()
+            predict_thr = np.atleast_1d(0.5)
         else:
-            predict_thr = np.atleast_1d(0)  # multiclass, single-label (argmax mode)
+            # multiclass, single-label (argmax mode)
+            predict_thr = np.atleast_1d(0)
     else:
         if np.ndim(predict_thr) > 1:
-            assert predict_thr.shape[0] == num_classes  # multi-label with custom thr vector
+            # multi-label with custom thr vector
+            assert predict_thr.shape[0] == num_classes
         else:
             if predict_thr == 0:
-                predict_thr = np.atleast_1d(0.5)  # binary or multi-label scalar default
+                # binary or multi-label scalar default
+                predict_thr = np.atleast_1d(0.5)
             else:
-                predict_thr = np.atleast_1d(predict_thr)  # user specified binary or multi-label scalar
+                # user specified binary or multi-label scalar
+                predict_thr = np.atleast_1d(predict_thr)
 
     # if predict_thr[0] == 0 and num_classes > 1:  # if scalar
     #     pthr = 0.5  # multiclass, single-label (argmax) or multilabel case default
@@ -118,7 +105,8 @@ def one_hot(truth: np.ndarray,
             predb = np.zeros(predict.shape, dtype=np.int8)  # frames x num_classes one-hot binary
             truthb = np.zeros(truth.shape, dtype=np.int8)  # frames x num_classes one-hot binary
             predb[np.arange(predb.shape[0]), plabel] = 1  # single-label frames x num_classes
-            truthb[np.arange(truthb.shape[0]), tlabel] = 1  # single-label frames x num_classes
+            if np.sum(truth):  # special case all zero truth leave tlabel all zeros
+                truthb[np.arange(truthb.shape[0]), tlabel] = 1  # single-label frames x num_classes
         else:  # multi-label prob threshold comparison (multiple classes)
             predb = np.int8(predict >= predict_thr.transpose())  # frames x num_classes multilabel one-hot decision
             truthb = np.int8(truth >= truth_thr)  # frames x num_classes multilabel one-hot decision
@@ -186,13 +174,16 @@ def one_hot(truth: np.ndarray,
         # Average Precision also called area under the PR curve AUCPR and
         # AUC ROC curve using binary-ized truth and continuous prediction probabilities
         # sklearn returns nan if no active truth in a class but w/un-suppressible div-by-zero warning
-        if np.sum(truthb[:, nci]) == 0:  # no active truth must be NaN
+        if np.sum(truthb[:, nci]) == 0:  # if no active classes both sklearn will fail, set to NaN
             AUC = np.NaN
             AP = np.NaN
             # threshold_optpr[nci] = np.NaN
         else:
             AP = average_precision_score(truthb[:, nci], predict[:, nci], average=None)
-            AUC = roc_auc_score(truthb[:, nci], predict[:, nci], average=None)
+            if len(np.unique(truthb[:, nci])) < 2:  # if active classes not > 1 AUC must be NaN
+                AUC = np.NaN  # i.e. all ones sklearn auc will fail
+            else:
+                AUC = roc_auc_score(truthb[:, nci], predict[:, nci], average=None)
             # # Optimal threshold from PR curve, optimizes f-score
             # precision, recall, thresholds = precision_recall_curve(truthb[:, nci], predict[:, nci])
             # fscore = (2 * precision * recall) / (precision + recall)
@@ -238,7 +229,10 @@ def one_hot(truth: np.ndarray,
         if np.sum(truthb):
             taidx = np.sum(truthb, axis=0) > 0
             wap = average_precision_score(truthb[:, taidx], predict[:, taidx], average='weighted')
-            wauc = roc_auc_score(truthb[:, taidx], predict[:, taidx], average='weighted')
+            if len(np.unique(truthb[:, taidx])) < 2:
+                wauc = np.NaN
+            else:
+                wauc = roc_auc_score(truthb[:, taidx], predict[:, taidx], average='weighted')
         else:
             wap = np.NaN
             wauc = np.NaN
