@@ -1,4 +1,3 @@
-from functools import partial
 from os.path import exists
 from os.path import expandvars
 from typing import List
@@ -11,6 +10,9 @@ from sonusai.mixture.apply_augmentation import apply_augmentation
 from sonusai.mixture.read_audio import read_audio
 from sonusai.utils import p_map
 
+# NOTE: multiprocessing dictionary is required for run-time performance; using 'partial' is much slower.
+MP_DICT = dict()
+
 
 def build_noise_audio_db(mixdb: dict, show_progress: bool = False) -> List[dict]:
     """Build a list of noise audio data.
@@ -52,14 +54,17 @@ def build_target_audio_db(mixdb: dict, show_progress: bool = False) -> list:
         'file'  file index
         'audio' audio samples
     """
+    MP_DICT['mixdb'] = mixdb
+
     indices = sorted(list(set([sub['target_file_index'] for sub in mixdb['mixtures']])))
     progress = tqdm(total=len(indices), desc='Read target audio', disable=not show_progress)
-    db = p_map(partial(_read_target_audio, mixdb=mixdb), indices, progress=progress)
+    db = p_map(_read_target_audio, indices, progress=progress)
     return db
 
 
-def _read_target_audio(file_index: int, mixdb: dict) -> dict:
+def _read_target_audio(file_index: int) -> dict:
     """Parallel target audio reader kernel."""
+    mixdb = MP_DICT['mixdb']
     return {
         'file':  file_index,
         'audio': read_audio(name=mixdb['targets'][file_index]['name'])
