@@ -1,14 +1,12 @@
+from copy import deepcopy
 from typing import List
 from typing import Union
 
+from sonusai import SonusAIError
+from sonusai.mixture.get_mixtures_from_mixid import get_mixtures_from_mixid
 
-def new_mixdb_from_mixid(mixdb: dict,
-                         mixid: Union[str, List[int]]) -> dict:
-    from copy import deepcopy
-
-    from sonusai import SonusAIError
-    from sonusai.mixture.get_mixtures_from_mixid import get_mixtures_from_mixid
 
+def new_mixdb_from_mixid(mixdb: dict, mixid: Union[str, List[int]]) -> dict:
     mixdb_out = deepcopy(mixdb)
     mixdb_out['mixtures'] = get_mixtures_from_mixid(mixdb_out, mixid)
 
