@@ -4,12 +4,16 @@ from sonusai.utils.expandvars import expandvars
 from sonusai.utils.grouper import grouper
 from sonusai.utils.human_readable_size import human_readable_size
 from sonusai.utils.int16_to_float import int16_to_float
+from sonusai.utils.onnx_utils import add_sonusai_metadata
+from sonusai.utils.onnx_utils import create_onnx_from_keras
+from sonusai.utils.onnx_utils import replace_stateful_grus
 from sonusai.utils.p_tqdm import p_map
 from sonusai.utils.p_tqdm import p_umap
 from sonusai.utils.read_feature_data import read_feature_data
 from sonusai.utils.read_mixture_data import read_mixture_data
 from sonusai.utils.read_predict_data import read_predict_data
-from sonusai.utils.reshape_inputs import reshape_inputs
+from sonusai.utils.reshape import reshape_inputs
+from sonusai.utils.reshape import reshape_truth_predict
 from sonusai.utils.seconds_to_hms import seconds_to_hms
 from sonusai.utils.to_numpy_array import to_numpy_array
 from sonusai.utils.trim_docstring import trim_docstring
